
// The following C++ code is translated from the Lisp code
// in ``Calendrical Calculations'' by Nachum Dershowitz and
// Edward M. Reingold, Software---Practice & Experience,
// vol. 20, no. 9 (September, 1990), pp. 899--928.

// This code is in the public domain, but any use of it
// should publically acknowledge its source.

// Classes GregorianDate, JulianDate, IsoDate, IslamicDate,
// and HebrewDate

#ifndef _CALENDARH
#define _CALENDARH

#include <iostream>
#include <string>

using namespace std;


int LastDayOfGregorianMonth(int month, int year);
int LastDayOfJulianMonth(int month, int year) ;
int LastDayOfIslamicMonth(int month, int year) ;
int LastDayOfHebrewMonth(int month, int year) ;

int IslamicLeapYear(int year);
int HebrewLeapYear(int year);


class GregorianDate {
private:
  int year;   // 1...
  int month;  // 1 == January, ..., 12 == December
  int day;    // 1..LastDayOfGregorianMonth(month, year)
  
public:
  GregorianDate(int m, int d, int y) { month = m; day = d; year = y; }
  
  GregorianDate(int d) ;
  
  operator int() ;
  
  int GetMonth() { return month; }
  int GetDay() { return day; }
  int GetYear() { return year; }

  friend ostream& operator<<(ostream& c, GregorianDate d) ;
};


class JulianDate {
private:
  int year;   // 1...
  int month;  // 1 == January, ..., 12 == December
  int day;    // 1..LastDayOfJulianMonth(month, year)
  
public:
  JulianDate(int m, int d, int y) { month = m; day = d; year = y; }
  
  JulianDate(int d);
  
  operator int() ;
  
  int GetMonth() { return month; }
  int GetDay() { return day; }
  int GetYear() { return year; }
  int absolute();

  friend ostream& operator<<(ostream& c, JulianDate d) ;
  
};

class IsoDate {
private:
  int year;  // 1...
  int week;  // 1..52 or 53
  int day;   // 1..7
  
public:
  IsoDate(int w, int d, int y) { week = w; day = d; year = y; }
  
  IsoDate(int d) ; 
  
  operator int() ;
  
  int GetWeek() { return week; }
  int GetDay() { return day; }
  int GetYear() { return year; }


  friend ostream& operator<<(ostream& c,  IsoDate d) ;  
};


class IslamicDate {
private:
  int year;   // 1...
  int month;  // 1..13 (12 in a common year)
  int day;    // 1..LastDayOfIslamicMonth(month,year)
  
public:
  IslamicDate(int m, int d, int y) { month = m; day = d; year = y; }
  
  IslamicDate(int d) ;
  
  operator int() ; 
  
  int GetMonth() { return month; }
  int GetDay() { return day; }
  int GetYear() { return year; }

  friend ostream& operator<<(ostream& c,  IslamicDate d) ;  
  
};


class HebrewDate {
private:
  int year;   // 1...
  int month;  // 1..LastMonthOfHebrewYear(year)
  int day;    // 1..LastDayOfHebrewMonth(month, year)
  
public:
  HebrewDate(int m, int d, int y) { month = m; day = d; year = y; }
  
  HebrewDate(int d) ;
  
  operator int() ;
  
  int GetMonth() { return month; }
  int GetDay() { return day; }
  int GetYear() { return year; }
  
  friend ostream& operator<<(ostream& c,  HebrewDate d) ;  

};


#endif
