
/*------------------------------------------------------------------------------
Hebrew Calendar is a simple program which show the hebrew dates and holydays
of the gregorian calendar

Copyright (C) 2005  Asaf Shachar <asafsh1@users.sourceforge.net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
------------------------------------------------------------------------------*/

#include <gtk/gtk.h>
#include "hagim.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gtklocale.h"

void 
export_hagim (gchar *filename, int year, int lang, int format)
{
    Hagim hagim_year;  

    hagim_year.cache_year (year);
    switch (format) {
	case 0:	
    	  hagim_year.export_ical (filename, lang);
	  break;
	default:
	  g_print ("Error: this format is not supported yet\n");
    }
}


gint 
dialog_confirm (GtkWidget *parent, gchar *message)
{

  GtkWidget *dialog;
  GtkWidget  *label;

  dialog = gtk_message_dialog_new (      GTK_WINDOW (parent),
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
					 GTK_MESSAGE_QUESTION,
                                         GTK_BUTTONS_OK_CANCEL,
                                         "%s",
                                         message
                                         );


  //label = gtk_label_new (message);
  //gtk_widget_show (label);	

  //gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);   
  
  gint result = gtk_dialog_run (GTK_DIALOG (dialog));
     	
  gtk_widget_destroy (dialog);

  return result;

}


gint
dialog_export (GtkWidget *parent, int year, int month, int day)
{

  GtkWidget *dialog;
  GtkWidget *buttons;	
  GtkWidget *optionsbox;

  GtkWidget *filechooser;
  gchar *filename;

  GtkWidget *langcombo;
  GtkWidget *langlabel;
  GtkWidget *langhbox;
  int lang_idx=1;

  GtkWidget *formatcombo;
  GtkWidget *formatlabel;
  GtkWidget *formatbox;
  int format_idx=0;

  GtkWidget vbox;


  dialog = gtk_dialog_new_with_buttons (_("Export"),
                                         GTK_WINDOW (parent),
                                         GTK_DIALOG_NO_SEPARATOR ,
                                         GTK_STOCK_CANCEL,
                                         GTK_RESPONSE_NONE,
                                         NULL);
  
  buttons = gtk_dialog_add_button      (GTK_DIALOG (dialog),
                                         _("Export"),
                                         1);

  
  filechooser=gtk_file_chooser_widget_new (GTK_FILE_CHOOSER_ACTION_SAVE);
  gtk_widget_show (filechooser);	
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), filechooser, TRUE, TRUE, 0);


  langhbox = gtk_hbox_new (FALSE, 5);
  langlabel= gtk_label_new (_("Language:"));
  langcombo =gtk_combo_box_new_text ();

  gtk_combo_box_append_text (GTK_COMBO_BOX (langcombo), _("English"));    
  gtk_combo_box_append_text (GTK_COMBO_BOX (langcombo), _("Hebrew"));    
  gtk_combo_box_set_active  (GTK_COMBO_BOX (langcombo), lang_idx);

  gtk_box_pack_start (GTK_BOX (langhbox), langlabel, FALSE, FALSE, 5);
  gtk_box_pack_start (GTK_BOX (langhbox), langcombo, FALSE, FALSE, 5);

  formatbox = gtk_hbox_new (FALSE, 5);
  formatlabel= gtk_label_new (_("Format:"));
  formatcombo =gtk_combo_box_new_text ();
  gtk_combo_box_append_text (GTK_COMBO_BOX (formatcombo), "iCal");    
  gtk_combo_box_set_active  (GTK_COMBO_BOX (formatcombo), format_idx);

  gtk_box_pack_start (GTK_BOX (formatbox), formatlabel, FALSE, FALSE, 5);
  gtk_box_pack_start (GTK_BOX (formatbox), formatcombo, FALSE, FALSE, 5);

  optionsbox=gtk_vbox_new (FALSE,5);	
  gtk_box_pack_start (GTK_BOX (optionsbox), langhbox, FALSE, FALSE, 5);	
  gtk_box_pack_start (GTK_BOX (optionsbox), formatbox, FALSE, FALSE, 5);	
  gtk_widget_show_all (optionsbox);		

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), optionsbox, TRUE, TRUE, 0);



  gint result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
  {
      case 1:
         filename=gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filechooser));
	 if (filename!=NULL) {
	    if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
		gchar message[100];
		g_snprintf (message, 100, _("file: '%s' already exists continue?"), filename); 
	
		gint rc=dialog_confirm (dialog, message)  ;
		if (rc!=GTK_RESPONSE_OK)
		  break;
	    }
	    
	    format_idx=gtk_combo_box_get_active  (GTK_COMBO_BOX (formatcombo));
  	    lang_idx=gtk_combo_box_get_active  (GTK_COMBO_BOX (langcombo));
	    export_hagim (filename, year, lang_idx, format_idx); 	
	    g_free (filename); 	
	 }
         break;
      default:
         
         break;
  }

  gtk_widget_destroy (dialog);
  return result;

}


guint dialog_pick_colors (GtkWidget *parent, char *bgcolors[3])
{
  
  GtkWidget *dialog;
  GtkWidget *buttons;	
  GtkWidget *vbox;
  GtkWidget *table;
  GtkWidget *label;
  GdkColor color;
  gchar *color_desc[3]={_("Current"), _("Halfday"), _("Holyday")};
  GtkWidget *color_widget[3];

  dialog = gtk_dialog_new_with_buttons (_("Pick Colors"),
                                         GTK_WINDOW (parent),
                                         GTK_DIALOG_NO_SEPARATOR ,
                                         GTK_STOCK_CANCEL,
                                         GTK_RESPONSE_NONE,
                                         NULL);
  
  buttons = gtk_dialog_add_button      (GTK_DIALOG (dialog),
                                         _("Apply"),
                                         1);

  table = gtk_table_new  (2,3,TRUE);


  for (int i=0;i<3;i++) 
  {
      gdk_color_parse (bgcolors[i], &color);   
      color_widget[i]=gtk_color_button_new_with_color (&color);

      gtk_widget_show (color_widget[i]);

      label=gtk_label_new (color_desc[i]);
     
      gtk_table_attach_defaults (GTK_TABLE (table), label,i,i+1,0,1 );    
      gtk_table_attach_defaults (GTK_TABLE (table), color_widget[i], 
			       i,i+1,1,2);    

      gtk_widget_show (label);
      gtk_widget_show (color_widget[i]);

  }

  gtk_widget_show_all (	table );		

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog)->vbox), table, FALSE, FALSE, 0);

  gint result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
  {
      case 1:
	for (int i=0;i<3;i++)
	{
	    gtk_color_button_get_color (GTK_COLOR_BUTTON (color_widget[i]), &color);
      	    g_snprintf (bgcolors[i], 8, "#%02X%02X%02X", 
	      color.red / 256, color.green / 256, color.blue / 256);
	}

	break;
      default:
	break;	
  }

  gtk_widget_destroy (dialog);
  return result;
}


void 
dialog_about ()
{

  GtkWidget *dialog;

  dialog = gtk_about_dialog_new ();		
  const char *authors[]={"Asaf Shahar", NULL};

  gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (dialog), VERSION);
  gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (dialog), "Copyrights Asaf Shahar (C)");
  gtk_about_dialog_set_license (GTK_ABOUT_DIALOG (dialog), "GPL - General Public License");
  gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (dialog), "http://hcal.sourceforge.net");


  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);

}
