/*------------------------------------------------------------------------------
Hebrew Calendar is a simple program which show the hebrew dates and holydays
of the gregorian calendar

Copyright (C) 2005  Asaf Shachar <asafsh1@users.sourceforge.net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
------------------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gtklocale.h"

#include <gtk/gtk.h>
#include "hebcalendar.h"


static void        
day_change (GtkWidget *widget, gpointer data)
{
  guint day;

  day=(guint)gtk_spin_button_get_value (GTK_SPIN_BUTTON (widget));
  if (day)
    heb_calendar_select_day  (HEBCALENDAR (data), day);


}

static void        
month_year_change (GtkWidget *widget, gpointer data)
{

  guint month;
  guint year;
  int object_id;

  object_id=GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "object_id"));
  g_object_get (HEBCALENDAR (data),
                "month", &month,
		"year", &year,
		NULL);
  
  if (object_id) {
    if (object_id==1)
      month=(guint)gtk_spin_button_get_value (GTK_SPIN_BUTTON (widget));
    else
      year=(guint)gtk_spin_button_get_value (GTK_SPIN_BUTTON (widget));

    heb_calendar_select_month (HEBCALENDAR (data), month, year);
  }

}



static GtkWidget *
create_spin (GtkWidget *hc, char *title, int start, 
	     int end, int init, gpointer callback_func, int id)
{
  GtkWidget *spin;
  GtkWidget *label;
  GtkWidget *hbox;

  spin =gtk_spin_button_new_with_range  (start, end, 1);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (spin), init);

  g_signal_connect (G_OBJECT (spin), "value-changed", 
		    G_CALLBACK (callback_func), gpointer (hc));

  g_object_set_data (G_OBJECT (spin), "object_id", GINT_TO_POINTER (id));

  gtk_widget_show (spin);

  hbox = gtk_hbox_new  (FALSE, 5);

  label=gtk_label_new (title);
  gtk_widget_show (label);

  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 5);
  gtk_box_pack_start (GTK_BOX (hbox), spin, TRUE, TRUE, 5);

  gtk_widget_show (hbox);
  
  return hbox;

}


GtkWidget *
create_month_year (GtkWidget *hc)
{

  GtkWidget *frame;
  GtkWidget *hbox;
  guint day,month,year;

  g_object_get (HEBCALENDAR (hc),
		"day", &day,
                "month", &month,
		"year", &year,
		NULL);

  hbox = gtk_hbox_new  (FALSE, 0);
  gtk_widget_show (hbox);

  gtk_box_pack_start (GTK_BOX (hbox),
		      create_spin (hc,_("Day:"),1,31,day,
				   (gpointer)day_change,0), 
		      FALSE, FALSE, 0); 

  gtk_box_pack_start (GTK_BOX (hbox),
		      create_spin (hc, _("Month:"),1,12,month,
				   (gpointer)month_year_change, 1), 
		      FALSE, FALSE, 0); 

  gtk_box_pack_start (GTK_BOX (hbox),
		      create_spin (hc, _("Year:"),1000,5000,year,
				   (gpointer)month_year_change, 2), 
		      FALSE, FALSE, 0); 


  frame= gtk_frame_new (_("Jump To:"));
  gtk_container_add (GTK_CONTAINER (frame), hbox);
  gtk_widget_show (frame);

  return frame;
}

