/*------------------------------------------------------------------------------
Hebrew Calendar is a simple program which show the hebrew dates and holydays
of the gregorian calendar

Copyright (C) 2005  Asaf Shachar <asafsh1@users.sourceforge.net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
------------------------------------------------------------------------------*/
#include <gtk/gtk.h>
#include <glib.h>
#include "eventssql.h"

static   GtkWidget *treeview;   
static   GtkWidget *text_new;

enum
{
  DATE_COLUMN,
  INDEX_COLUMN,
  CHECKED_COLUMN,
  DESC_COLUMN,
  N_COLUMNS
};


static void        
text_enter_callback   (GtkEntry *entry, gpointer user_data)
{
  const gchar  *text;
  guint        date;
  gint         index;
  GtkTreeModel *model;
  GtkTreeIter  iter1;
  
  text = gtk_entry_get_text  (GTK_ENTRY (entry));
  model=gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
  date=GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (model), "date"));

  if ( (index= sql_insert_event (date, 0, (gchar *)text)) ) {
    gtk_list_store_append (GTK_LIST_STORE (model), &iter1);  
    gtk_list_store_set (GTK_LIST_STORE (model), &iter1,
		      DATE_COLUMN, date,		      
		      INDEX_COLUMN, index,
		      CHECKED_COLUMN, 0,
		      DESC_COLUMN, text,
		      -1);
    gtk_entry_set_text (GTK_ENTRY (entry), "");
  }

}


static void        
text_edited_callback (GtkCellRendererText *cell,
                              gchar               *path,
                              gchar               *new_text,
                              gpointer             user_data)
{

   GtkTreeModel *model;
   GtkTreePath  *treepath;
   GtkTreeIter  iter;
   guint        data_date, data_index, data_checked;


   model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
   treepath = gtk_tree_path_new_from_string  (path);
   if (treepath!=NULL) {
       if (gtk_tree_model_get_iter(GTK_TREE_MODEL (model),
                                   &iter,
				   treepath) ) {
	 

	 gtk_tree_model_get (GTK_TREE_MODEL (model),
                             &iter,
			     DATE_COLUMN, &data_date,
			     INDEX_COLUMN, &data_index,
			     CHECKED_COLUMN, &data_checked,
			     -1);

	 if ( sql_update_event (data_date,data_index,data_checked,new_text))
           gtk_list_store_set (GTK_LIST_STORE (model), &iter,
                   DESC_COLUMN, new_text,
                   -1);
	 else
	   //error
	   g_print ("insert data failed!\n");
       }

     gtk_tree_path_free (treepath);
   }

}


static void 
checked_toggled  (GtkCellRendererToggle *cell_renderer,
                              gchar *path,
                              gpointer user_data)
{
   gboolean     checked;
   GtkTreeModel *model;
   GtkTreePath  *treepath;
   GtkTreeIter  iter;
   guint        data_date, data_index, data_checked;
   gchar        *data_desc;


   model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
   treepath = gtk_tree_path_new_from_string  (path);
   if (treepath!=NULL) {
       if (gtk_tree_model_get_iter(GTK_TREE_MODEL (model),
                                   &iter,
				   treepath) ) {
	 

 	  gtk_tree_model_get (GTK_TREE_MODEL (model),
                             &iter,
			     DATE_COLUMN, &data_date,
			     INDEX_COLUMN, &data_index,
			     CHECKED_COLUMN, &data_checked,
			     DESC_COLUMN, &data_desc,
			     -1);

	  // set to true or false by old value
	  checked=!data_checked;
	 
	  if ( sql_update_event (data_date,data_index,checked,data_desc)) {
	    gtk_list_store_set (GTK_LIST_STORE (model), &iter,
                   CHECKED_COLUMN, checked,
                   -1);
	  } else
	    // error
	    g_print ("error update data failed\n");

 	  g_free (data_desc);
       }

       gtk_tree_path_free (treepath);
   }

}


static gboolean
delete_button_press_callback (GtkObject *eventbox, 
                        GdkEventButton *event,
                         gpointer user_data)
{

  GtkTreeModel     *model;
  GtkTreeSelection *selection;
  GtkTreeIter      iter;
  guint            data_date, data_index;
  gchar            *data_desc;


  selection=gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
  model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));

  if (! gtk_tree_selection_get_selected (selection,
                                         &model,
                                         &iter))
    return TRUE;


  gtk_tree_model_get (GTK_TREE_MODEL (model),
                             &iter,
			     DATE_COLUMN, &data_date,
			     INDEX_COLUMN, &data_index,
			     DESC_COLUMN, &data_desc,
			     -1);

  if (sql_delete_event (data_date, data_index)) {
    gtk_list_store_remove  (GTK_LIST_STORE ( model ), &iter);
    gtk_entry_set_text (GTK_ENTRY (text_new), data_desc);   
    gtk_editable_select_region  (GTK_EDITABLE (text_new), 0, -1);
  } else
    //error
    g_print ("error delete data failed!\n");

  g_free (data_desc);

  return TRUE;

}



static gboolean
sort_button_press_callback (GtkObject *eventbox, 
                        GdkEventButton *event,
                         gpointer user_data)
{
  GtkTreeModel     *model;
  GtkSortType      order,neworder;
  gint             sortid;
  gboolean         sorted;
 
  model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));

  sorted=gtk_tree_sortable_get_sort_column_id ( GTK_TREE_SORTABLE (model), &sortid, &order);

  if (sorted && sortid==DESC_COLUMN)
    neworder=(order == GTK_SORT_ASCENDING) ? GTK_SORT_DESCENDING : 
      GTK_SORT_ASCENDING;
  else
    neworder=GTK_SORT_ASCENDING;

  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model),
                                        DESC_COLUMN, neworder);

  return TRUE;
}



void
create_tree_view ( )
{

   GtkTreeViewColumn *column;
   GtkCellRenderer   *renderer;
   GtkListStore      *store;


   store = gtk_list_store_new (N_COLUMNS,
			       G_TYPE_UINT,    // date
			       G_TYPE_INT,     // index
                               G_TYPE_BOOLEAN, // check
                               G_TYPE_STRING );// desc
  
   treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL ( store) );
   g_object_unref (store);      

   renderer = gtk_cell_renderer_toggle_new (); 
   column = gtk_tree_view_column_new_with_attributes ("Checked", renderer,
						      "active", CHECKED_COLUMN,
						      NULL);
   g_signal_connect (G_OBJECT (renderer), "toggled",	
		      G_CALLBACK (checked_toggled), NULL);

   gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);


   
   renderer = gtk_cell_renderer_text_new (); 
   column = gtk_tree_view_column_new_with_attributes ("Event",
                                                      renderer,
                                                      "text", DESC_COLUMN,
                                                      NULL);
   g_signal_connect(G_OBJECT (renderer), "edited", 
		    G_CALLBACK (text_edited_callback), 
		    NULL);   

   g_object_set (G_OBJECT (renderer),
                 "editable", TRUE,
                 NULL);


   gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

   /*
   renderer = gtk_cell_renderer_text_new (); 
   column = gtk_tree_view_column_new_with_attributes ("Index",
						      renderer,
						      "text", 
						      INDEX_COLUMN,NULL);

   gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
   */
   
   gtk_tree_view_set_rules_hint    (GTK_TREE_VIEW ( treeview ), TRUE );
   gtk_tree_view_set_headers_visible  (GTK_TREE_VIEW ( treeview ), FALSE );
                                                                               
}


GtkWidget *
create_data_view ( )
{

  GtkWidget *view;
  GtkWidget *vscrollbar;
  GtkWidget *data;
  
  view = gtk_viewport_new (NULL,NULL);
  gtk_viewport_set_shadow_type (GTK_VIEWPORT (view), GTK_SHADOW_ETCHED_IN);
  vscrollbar = gtk_vscrollbar_new  (gtk_viewport_get_vadjustment 
				    (GTK_VIEWPORT (view)));

  gtk_container_add (GTK_CONTAINER (view), treeview);  

  data =  gtk_hbox_new  (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (data), view, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (data), vscrollbar, FALSE, FALSE, 0);
  gtk_widget_set_size_request (GTK_WIDGET (data), 100,70);

  gtk_widget_show ( treeview );
  gtk_widget_show ( view );
  gtk_widget_show ( vscrollbar );
  gtk_widget_show ( data );

  return data;

}


GtkWidget *
create_data_panel ( )
{


  GtkWidget *deleteimage;
  GtkWidget *deletebox;
  GtkWidget *sortimage;
  GtkWidget *sortbox;

  GtkWidget *panel;

  deleteimage= gtk_image_new_from_stock ( "gtk-cancel", 
				     GTK_ICON_SIZE_SMALL_TOOLBAR);

  deletebox =  gtk_event_box_new ( );
  gtk_container_add (GTK_CONTAINER (deletebox), deleteimage);

  g_signal_connect (G_OBJECT (deletebox),"button_press_event" , 
 		    G_CALLBACK (delete_button_press_callback), NULL);

  gtk_widget_show (deleteimage);
  gtk_widget_show (deletebox);


  sortimage= gtk_image_new_from_stock ( "gtk-sort-ascending", 
				     GTK_ICON_SIZE_SMALL_TOOLBAR);

  sortbox =  gtk_event_box_new ( );
  gtk_container_add (GTK_CONTAINER (sortbox), sortimage);

  g_signal_connect (G_OBJECT (sortbox),"button_press_event" , 
 		    G_CALLBACK (sort_button_press_callback), NULL);

  gtk_widget_show (sortimage);
  gtk_widget_show (sortbox);


  panel=gtk_hbox_new (0, FALSE);
  gtk_box_pack_start (GTK_BOX (panel), deletebox, FALSE, FALSE, 5);
  gtk_box_pack_start (GTK_BOX (panel), sortbox, FALSE, FALSE, 5);
  
  gtk_widget_show (panel);


  return panel;

}


void 
populate_tree_model ( guint date )
{
    GtkTreeModel *model;
    GtkTreeIter  iter1;
    event_item   *data;
    
    model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));  
    g_object_set_data (G_OBJECT (model), "date", GUINT_TO_POINTER (date));
    

    sql_query_events ( date );    
    data=(event_item *)init_event_iterator ();
    gtk_list_store_clear ( GTK_LIST_STORE (model) );

    while (data!=NULL) {
	  gtk_list_store_append (GTK_LIST_STORE (model), &iter1);  
	  gtk_list_store_set (GTK_LIST_STORE (model), &iter1,
			      DATE_COLUMN, data->date,		      
			      INDEX_COLUMN, data->index,
			      CHECKED_COLUMN, data->status,
			      DESC_COLUMN, data->description,
			      -1);

	  data=event_iterator_next ();
    }

    free_events_data ();
}



GtkWidget *
create_data_widget ( )
{

  GtkWidget *vbox;
  GtkWidget *data;
  GtkWidget *panel;

  create_tree_view ( ) ;
  text_new=gtk_entry_new_with_max_length (100);
  //  gtk_entry_set_has_frame (GTK_ENTRY (text), FALSE);
  g_signal_connect (G_OBJECT (text_new), "activate",	
		      G_CALLBACK (text_enter_callback), NULL);
  gtk_widget_show (text_new);

  data=create_data_view ( );
  panel=create_data_panel ( );

  vbox=gtk_vbox_new (0, FALSE);
  gtk_box_pack_start (GTK_BOX (vbox), panel, FALSE, FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), data, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), text_new, FALSE, FALSE, 0);


  gtk_widget_show (vbox);


  return vbox;

}


