/*------------------------------------------------------------------------------
Hebrew Calendar is a simple program which show the hebrew dates and holydays
of the gregorian calendar

Copyright (C) 2005  Asaf Shachar <asafsh1@users.sourceforge.net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
------------------------------------------------------------------------------*/
#define _DEBUG 0

#include <string.h>
#include <sqlite3.h>
#include <glib.h>
#include "sqlitedb.h"
#include "eventssql.h"

GList *event_list;
GList *event_iterator;

static char *events_sql="select date,event_index,status,descreption from userevents ";


/*--------------------------------------------------------------
sql query callbacks
--------------------------------------------------------------*/

static 
int callback_query(void *t, int argc, char **argv, char **azColName)
{
  
  event_item *data;

  if (argc>3) {
    data=g_new (event_item, 1);

    data->date=(guint) g_strtod (argv[0],(gchar**)NULL);
    data->index=(guint) g_strtod (argv[1],(gchar**)NULL);
    data->status=(guint) g_strtod (argv[2],(gchar**)NULL);
    data->description=g_strdup (argv[3]);

    event_list=g_list_append (event_list, data);
  }

  return 0;

}



static 
int callback_query_month (void *t, int argc, char **argv, char **azColName)
{

  int day;
  int *month=(int *)t;

  if (argc) {
    day=argv[0] ? (guint) g_strtod (argv[0],(gchar**)NULL) : 0;
    if (day)
      month[day-month[0]+1]=1;
  }
    

  return 0;

}


static 
int callback_max(void *t, int argc, char **argv, char **azColName)
{

  int *max=(int *)t;

  if (argc)
    *max=argv[0] ? (guint) g_strtod (argv[0],(gchar**)NULL) : 0;
  else
    *max=0;

  return 0;

}


/*--------------------------------------------------------------
events data query result functions
--------------------------------------------------------------*/

void 
free_events_data ( )
{
  GList *items;
  event_item *data;

  if (event_list==NULL) {
    if (_DEBUG) g_print ("Empty\n");
    return;
  }

  items=g_list_first (event_list);
  while (items!=NULL) {
      data=(event_item *)items->data;
      if (_DEBUG)
	g_print ("free %d|%d|%d|%s\n", data->date, data->index, 
		 data->status, data->description);

      g_free (data->description);
      g_free (data);
      items=(GList *)g_list_next (items);
  }

  g_list_free (event_list);
  event_list=NULL;

}


event_item *
init_event_iterator ()
{
  if ((event_iterator=g_list_first (event_list))==NULL)
    return NULL;

  return (event_item *)event_iterator->data;
}


event_item *event_iterator_next ()
{
  if ((event_iterator=g_list_next (event_iterator))==NULL)
    return NULL;
  
  return (event_item *)event_iterator->data;
}


void 
print_events_data ()
{
  event_item *data;

  data=(event_item *)init_event_iterator ();
  while (data!=NULL) {
    g_print ("iterator %d|%d|%d|%s\n", data->date, data->index, 
		 data->status, data->description);
    data=event_iterator_next ();
  }

}

/*--------------------------------------------------------------
sql query update and insert functions
--------------------------------------------------------------*/
int
sql_insert_event (guint date, guint acc, gchar *desc)
{
  char *statement;
  int rc;
  int max_index;

  statement=sqlite3_mprintf ("SELECT MAX (event_index) FROM userevents WHERE date=%d", date);
  if ((rc=sqlite_exec_query (callback_max, statement, (void *)&max_index)))
    if (_DEBUG) 
	g_print ("%d\n", max_index);

  sqlite3_free(statement);    
  
  if (rc) {
    statement=sqlite3_mprintf ("INSERT INTO userevents VALUES(%d,%d,%d,%Q)",
		     date, max_index+1, acc, desc );
    if ((rc=sqlite_exec_query (NULL, statement, NULL)))
      if (_DEBUG) g_print ("insert ok\n");
 
    sqlite3_free(statement);
    // set the return code to index
    rc=max_index + 1;
  }
  
  return rc;
}


int
sql_update_event (guint date, guint index, guint acc, gchar *desc)
{

  char *statement;
  int rc;

  if (index) 
    statement=sqlite3_mprintf ("UPDATE userevents SET status=%d,descreption=%Q WHERE date=%d AND event_index=%d;", acc, desc, date, index);
  else 
    statement=sqlite3_mprintf ("UPDATE userevents SET status=%d,descreption=%Q WHERE date=%d;", acc, desc, date, index);


  if (_DEBUG)
    g_print ("%s\n", statement);

  if ( (rc = sqlite_exec_query (NULL, statement, NULL)) ) {
    if (_DEBUG) g_print ("update ok\n");
  }

  sqlite3_free (statement);

  return rc;
}


int
sql_delete_event ( guint date, guint index )
{

  char *statement;
  int rc;

  statement=sqlite3_mprintf ("DELETE FROM userevents WHERE date=%d AND event_index=%d ;",  date, index);

  if (_DEBUG)
    g_print ("%s\n", statement);

  if ( (rc = sqlite_exec_query (NULL, statement, NULL)) ) {
    if (_DEBUG) g_print ("delete ok\n");
  }

  sqlite3_free (statement);

  return rc;
}



int
sql_query_events ( guint date )
{
  gchar *statement;
  int rc;

  statement=sqlite3_mprintf ("%q WHERE date>%d AND date<%d",
			     events_sql,date-1,date+1 );
  if (_DEBUG)
    g_print ("%s\n", statement);

  if ( (rc = sqlite_exec_query (callback_query, statement, NULL)) ) {
    if (_DEBUG) g_print ("query ok\n");
  }

  sqlite3_free (statement);

  return rc;

}



int
sql_query_month_events ( guint date, int month[32] )
{
  gchar *statement;
  int rc,i;
  
  for (i=0;i<32;i++)
    month[i]=0;

  month[0]=date;

  statement=sqlite3_mprintf ("%q WHERE date>%d AND date<%d",
			     events_sql,date-1,date+31 );
  if (_DEBUG)
    g_print ("%s\n", statement);

  if ( (rc = sqlite_exec_query (callback_query_month, statement, (void *)month)) ) {
    if (_DEBUG) g_print ("query ok\n");
  }

  sqlite3_free (statement);

  return rc;

}

