/*------------------------------------------------------------------------------
Hebrew Calendar is a simple program which show the hebrew dates and holydays
of the gregorian calendar

Copyright (C) 2005  Asaf Shachar <asafsh1@users.sourceforge.net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
------------------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef WITH_LIBHDATE
#include <hdatepp.h>
using namespace hdate;
#endif

#include <fstream>
#include "hagim.h"
#include "tohebalef.h"



static hagimdate thag[] = {{ 9,25, 0, 0},{ 11, 15, 0 , 0},{ 1, 15, 0, 0},
		    { 2, 5, 0, 0},{ 12, 14, 0 , 0},{ 7 , 1 ,1 ,0}, 
		    { 9,25, 1, 0} };




static char *hag_edesc[] = {"chanuka", "to-b'shvat", "pesach", "lag-b'omer","shavuot", 
			   "yom-hazikaron", "yom-haatzmaut", "purim", "rosh-hashana", 
			   "yom-kipur", "sukkut", "simhat-tora" };

static char *hag_hdesc[] = { "חנוכה", "טו׳ בשבט", "פסח", "לג׳ בעומר", "שבועות", 
		     "יום הזיכרון", "יום העצמאות", "פורים", "ראש השנה", 	
		     "יום כיפור", "סוכות", "שמחת תורה" 
			   } ;
static char *erev = "ערב";
static char *holmoed =  "חוה׳מ";
static char *hosha= "הושענא רבה";

static char *ical_headers = 
"BEGIN:VCALENDAR\nVERSION:2.0\nCALSCALE:GREGORIAN\nMETHOD:PUBLISH"
;

static char *ical_footers =  
"END:VCALENDAR"
;

void print_ical_event (ofstream &out, GregorianDate &gd, string &summery)
{
  char date[9];
  sprintf (date, "%d%02d%02d", gd.GetYear(), gd.GetMonth(), gd.GetDay());
  date[8]=0;	

  out << "BEGIN:VEVENT\n";
  out << "DTSTART;VALUE=DATE:" << date << "\n";
  out << "SUMMARY:" << summery << "\n"; 
  out << "CLASS:PUBLIC\n";
  out << "DTEND;VALUE=DATE:" << date << "\n";
  out << "END:VEVENT\n";
}


string gethosha () { return hosha; }

void geterev (string &hag)
{
  hag.insert (0, " ");
  hag.insert (0, erev);
}

void getday (string &hag, int n, int moed)
{
  string hnum=tohebalefgeresh (n);
  hnum+=" ";
  if (moed) {
     hnum+=holmoed;
     hnum+=" ";
  }

  hag.insert (0, hnum);
}

void tohebhag (string &d, int code, int daynum)
{

#ifdef WITH_LIBHDATE
  // fix me: add here hag desctription to d code is hdate code daynum is always zero
  return ;
  	
#else


  switch (code) {
	  case 0: //chanuka
	    getday (d, daynum, 0);
	    break;
	  case 8: // rosh hshana
	    if (!daynum)
		geterev (d);
	    else 
		getday (d, daynum, 0);
	    break;
	  case 2: // pesach
	  case 10: // sukkot
	    if (!daynum) 
		geterev (d);
	    else if (daynum > 1 && daynum < 7) 
		getday (d, daynum - 1, 1);
	    else if (daynum == 7) 
		if (code==2)
		  getday (d, daynum, 0);	    
		else 
 		  d=gethosha ();
	    break;
	  case 9: //yom kipur
	  case 5: //yom hzikaron
	  case 4: //shavuot
	    if (!daynum)
		geterev (d);
	    break;	  
  }

#endif

}

void Hagim::addhag (int code, int adstart, int adend, int offset)
{
    struct hag_desc hd;

    for (int i=adstart;i<adend;i++)
    {
      hd.code=code;
      hd.desc=hag_edesc[code];
      hd.hdesc=hag_hdesc[code];
      hd.daynum=i-adstart + offset + 1;	
      hagim[i]=hd;
    }

}

int Hagim::get_hag (int ad, char *hag_desc, char *hag_hdesc, int *code, int *daynum)
{
    struct hag_desc hag=hagim[ad];

    if (hag.desc!="") {
	strcpy (hag_desc, hag.desc.c_str());
	strcpy (hag_hdesc, hag.hdesc.c_str());
	*code=hag.code;
	*daynum=hag.daynum;
	return true;
    }
    return false;
}

void Hagim::empty_cache ()
{
    hagim.erase (hagim.begin(),hagim.end());
    cached=false;
}

void Hagim::print_cache ()
{
    hagim_cache::const_iterator p;
    hag_desc d;
    for (p=hagim.begin ();p!=hagim.end (); p++) {
	GregorianDate gd (p->first);
	cout << p->first << " " << gd << " ";
	d=p->second;
	cout << d.code << " " << d.desc << " " << d.daynum <<  "\n";
    }
}

void Hagim::export_ical (char *file, int lang)
{
    
    ofstream output (file);
    
    output << ical_headers << "\n";

    hagim_cache::const_iterator p;
    hag_desc d;
    for (p=hagim.begin ();p!=hagim.end (); p++) {
	d=p->second;
  	GregorianDate gd (p->first);

#ifdef WITH_LIBHDATE
        print_ical_event (output, gd, d.desc);
#else
	if (d.daynum) {
	  if (lang==1) {
	    string hag=d.hdesc;
	    tohebhag (hag, d.code, d.daynum);		
	    print_ical_event (output, gd, hag);
	  } else
	    print_ical_event (output, gd, d.desc);
	}
#endif

    }

    output << ical_footers << "\n" ;    	

    output.close ();	

}

void Hagim::cache_year (int y)
{
    GregorianDate g(1,1,y);

    int a = g;

#ifdef WITH_LIBHDATE

    Hdate h;

    h.set_gdate (1, 1, y);
    	
    int jd=h.get_julian ();
    int offset=a-jd;
	
    for (int i=0;i<365;i++) {
      h.set_jd (jd);
      if ( int d=h.get_holyday () ) {
	struct hag_desc hd;

        hd.code=d;
	hd.desc=h.get_holyday_string (d);
	hd.daynum=1;
	hagim[jd+offset]=hd;
      }

      jd++;
       	
    }

#else

    HebrewDate h(a);
    int starty=a;

    GregorianDate e(1,1,y+1);
    int endy=e;

    // set month leap year for purim
    if (HebrewLeapYear (h.GetYear()))
    	thag[4].mleap=1;
    else
	thag[4].mleap=0;

    for (int i=0; i<7; i++) {
      HebrewDate h0(thag[i].m + thag[i].mleap, thag[i].d, thag[i].yleap + h.GetYear ());
      a=h0;
      switch (i) {
      case 0:
      case 6:
	//"chanuka"
	if (a<starty && i==0)
	  addhag (0, starty, min (a+8, endy), starty - a);	
	else
	  addhag (0, a, min (a+8, endy), 0);    
	break;
      case 1:
	//to-b'shvat
	//HebrewDate h1(11,15,h.GetYear ());

	addhag (1, a, a+1, -1);
	break;
      case 2: 
	//"pesach", "lag-b'omer","shavuot"
	//HebrewDate h2(1,15,h.GetYear ());

	addhag (2, a - 1, a+7, -1);
	addhag (3, a+33, a+34, -1);
	addhag (4, a+49, a+51, -1);
	break;
      case 3:
	//"yom-hazikaron", "yom-haatzmaut" 

	if (a%7==6) a-=2; // shabat
	else if (a%7==5) a-=1; // shishi
	else if (a%7==1) a+=1; // sheni

	addhag (5, a-2, a, -1);
	addhag (6, a, a + 1, 0);
	break;
      case 4: 
	//"purim"
	//HebrewDate h3(12+HebrewLeapYear (h.GetYear()),14,h.GetYear ());

	addhag (7, a, a+1, -1);
	break;
      case 5:
	//"rosh-hashana", "yom-kipur", "sukkut", "simhat-tora"
	//HebrewDate h4(7,1,h.GetYear ()+1);

	addhag (8, a - 1, a+2, -1);
	addhag (9, a+8, a+10, -1);
	addhag (10, a+13, a+21, -1);
	addhag (11, a+21, a+22, 0);
        break;
      }
    }

#endif

    cached=true;
}
