/*------------------------------------------------------------------------------
Hebrew Calendar is a simple program which show the hebrew dates and holydays
of the gregorian calendar

Copyright (C) 2005  Asaf Shachar <asafsh1@users.sourceforge.net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
------------------------------------------------------------------------------*/

#include <gtk/gtksignal.h>
#include <gtk/gtktable.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkarrow.h>
#include <gtk/gtkeventbox.h>
#include <gtk/gtkentry.h>
#include <gtk/gtktable.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtkhseparator.h>
#include <gtk/gtkimage.h>

#include <langinfo.h>
#include <time.h>


#include "hebcalendar.h"

// signals
enum {
  DAY_SELECTED_SIGNAL,
  MONTH_CHANGED_SIGNAL,
  MONTH_PREV_SIGNAL,
  MONTH_NEXT_SIGNAL,
  YEAR_PREV_SIGNAL,
  YEAR_NEXT_SIGNAL,
  LAST_SIGNAL
};

// arguments
enum {
  ARG_0,              /* Skip 0, an invalid argument ID */
  ARG_DAY,
  ARG_MONTH,
  ARG_YEAR,
  ARG_CURRENT_DAY_COLOR,
  ARG_EVENT_COLOR_HAG,
  ARG_EVENT_COLOR_HAGDAY
};

/* constants */
static char *boldmarkup	        ="<span size=\"medium\" weight=\"bold\">";
static char *normalmarkup	="<span size=\"medium\">";
static char *endmarkup		="</span>";

static char *currentcolor	="#46869C";
static char *hagcolor	        ="#DFDE2C";
static char *hagdaycolor	="#F2EE28";
//static char *userevent		="#99CE00";

static int localemonth[]={MON_1,MON_2,MON_3,MON_4,MON_5,MON_6,MON_7,MON_8,MON_9,MON_10,MON_11,MON_12};


/* callbacks */
static gboolean
month_button_press_callback (GtkObject *eventbox, 
			     GdkEventButton *event,
			     Hebcalendar *hc);

static gboolean
day_button_press_callback (GtkObject *eventbox, 
		       GdkEventButton *event,
                       Hebcalendar *hc);

static gboolean
to_current_date_callback (GtkObject *eventbox, 
		       GdkEventButton *event,
                       Hebcalendar *hc);

/*------------------------------------------------------------------------------
settings labels for hebrew month and year, month table, current hebrew date,
gergorian month and year

------------------------------------------------------------------------------*/

void 
setmonthyearlabel ( Hebcalendar *hc )
{

  gchar text[100], monthname[20], hebyear[10];

  heb_monthyear (&hc->hm, monthname);
  g_snprintf (text, 100, "%s%s%s",boldmarkup, monthname, endmarkup);
  gtk_label_set_markup (GTK_LABEL (hc->hebmonthlabel), text);
  
  heb_year (&hc->hm, hebyear); 
  g_snprintf (text, 100, "%s%s%s", boldmarkup, hebyear, endmarkup);
  gtk_label_set_markup (GTK_LABEL (hc->hebyearlabel), text);

}


void 
setmonthlabels ( Hebcalendar *hc)
{
  int daysnum;
  int i;
  int day;
  int event;
  char chebdate[256]; 
  gchar text[256], hebday[5];
  gchar desc[256];
  GdkColor color;
  Hebmonth *hm;

 
  hm=&(hc->hm);
  daysnum = hm->numofdays + hm->firstdow;
  for (i=0;i<42;i++) 
    if (i < daysnum  && i >= hm->firstdow ) {
      // a valid month day
      day= i - hm->firstdow ;

      heb_day (hm, day, hebday);

      if ( hc->daysmarkday[day] )
	g_snprintf (text, 100, "%s%d %s%s",boldmarkup, day+1, hebday, endmarkup);
      else
	g_snprintf (text, 100, "%s%d %s%s",normalmarkup, day+1, hebday, endmarkup);


      gtk_label_set_markup (GTK_LABEL (hc->dayslabel[i]), text);
      
      
      if ( (event = heb_hag (hm, day, desc)) ) {
	// it is a holiday
        if (event>1) 
	  // adjust for hag days > 1 see eventscolor table
	  event=2; 

	gtk_tooltips_set_tip (GTK_TOOLTIPS (hc->daystooltip[i]), 
			      hc->daysevent[i], desc, "");

	gtk_tooltips_enable (GTK_TOOLTIPS (hc->daystooltip[i]));
      } else {
	// disable old tooltips
	gtk_tooltips_disable (GTK_TOOLTIPS (hc->daystooltip[i]));
      }	    

      if (day+1==hm->gmday) {
	// set current day color

	heb_date (hm, chebdate);
	// set current day text
        gdk_color_parse (hc->currentdaycolor, &color);       
	gtk_entry_set_text (GTK_ENTRY (hc->texthebdate), chebdate);

      } else {
	// normal event colors 
       	gdk_color_parse (hc->eventcolors[event], &color);              
      }	

      gtk_widget_modify_bg (hc->daysevent[i], GTK_STATE_NORMAL, &color);
      g_object_set_data  (G_OBJECT (hc->daysevent[i]), "dayint",
			  GINT_TO_POINTER (day+1));   
    } else {
      // not a valid day
      gtk_label_set_markup (GTK_LABEL (hc->dayslabel[i]), "");
      g_object_set_data  (G_OBJECT (hc->daysevent[i]), "dayint", 
			  GINT_TO_POINTER (0)); 

      gtk_tooltips_disable (GTK_TOOLTIPS (hc->daystooltip[i]));
      gdk_color_parse ("white", &color);       
      gtk_widget_modify_bg (hc->daysevent[i], GTK_STATE_NORMAL, &color);

    }

}

void 
setmonthheadingslabel ( Hebcalendar *hc )
{
  char text[100];
  Hebmonth *hm;
  int i;
  GdkColor color;

  // set haedings color
  gdk_color_parse (hc->currentdaycolor, &color);     
  for (i=0; i<5; i++)
     gtk_widget_modify_bg (hc->weekdays[i], GTK_STATE_NORMAL, &color);
 
  hm=&hc->hm;
  g_snprintf (text, 100, "%s%s%s",normalmarkup, nl_langinfo (localemonth[hm->gmonth - 1 ]), endmarkup);
  gtk_label_set_markup (GTK_LABEL (hc->headmonthlabel), text);
 
  g_snprintf (text, 100, "%s%d%s",normalmarkup, hm->gyear, endmarkup);
  gtk_label_set_markup (GTK_LABEL (hc->headyearlabel), text);
  
}



void 
hebsetmonth ( Hebcalendar *hc)
{

  setmonthlabels (hc);
  setmonthyearlabel (hc);
  setmonthheadingslabel (hc);

}


void 
setcurrentdate ( Hebcalendar *hc )
{

  int month;
  int year;
  int mday;
  time_t curtime;
  struct tm *loctime;

  /* Get the current time.  */
  curtime = time (NULL);

  /* Convert it to local time representation.  */
  loctime = localtime (&curtime);
  
  month=loctime->tm_mon + 1;  // (0..11)
  year=loctime->tm_year+1900; // since 1900
  mday=loctime->tm_mday;

  hebmonth_init (&hc->hm, mday, month, year);

}


/*------------------------------------------------------------------------------
create the calendar gui:

  table titles,  month table,  month headings 
  hebrew date, month year, today button

------------------------------------------------------------------------------*/

void
create_daynames (Hebcalendar *hc)
{
  GdkColor color;
  GtkWidget *label; 
  gchar text[100], daybuf[5];
  int i;

  hc->daynames = gtk_hbox_new (FALSE, 0); 

  for (i=1;i<8;i++)
  {
    if (i==7)  // shin is 300 
      heb_alef (300, daybuf);
    else
      heb_alef (i, daybuf);

    g_snprintf (text, 100, "%s%s'%s", boldmarkup, daybuf, endmarkup);

    label=gtk_label_new (daybuf);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
    gtk_widget_set_size_request     (GTK_WIDGET (label), 45, 20);
    gtk_label_set_markup (GTK_LABEL (label), text);
    hc->weekdays[i-1] = gtk_event_box_new ();

    // is weekend (friday, suterday)
    if (i<6) {
      gdk_color_parse ("white", &color);
      gtk_widget_modify_fg (label, GTK_STATE_NORMAL, &color);
      gdk_color_parse (currentcolor, &color);
      gtk_widget_modify_bg (hc->weekdays[i-1], GTK_STATE_NORMAL, &color);
    } else {
      gdk_color_parse ("red", &color);
      gtk_widget_modify_fg (label, GTK_STATE_NORMAL, &color);
      gdk_color_parse ("white", &color);
      gtk_widget_modify_bg (hc->weekdays[i-1], GTK_STATE_NORMAL, &color);
    }
  
    gtk_container_add (GTK_CONTAINER (hc->weekdays[i-1]), label); 
    gtk_widget_show (label);
    gtk_widget_show (hc->weekdays[i-1]);

    gtk_box_pack_start (GTK_BOX (hc->daynames), hc->weekdays[i-1] , TRUE, TRUE, 0);
  }

}


void 
create_table (Hebcalendar *hc )
{
  GdkColor color;
  int i;

  gdk_color_parse ("white", &color);
  hc->table = gtk_table_new  (6,7,TRUE);

  for (i=0;i<42;i++) {
    hc->dayslabel[i]=gtk_label_new ("");

    gtk_label_set_justify (GTK_LABEL (hc->dayslabel[i]), GTK_JUSTIFY_RIGHT); 
    gtk_widget_set_size_request     (GTK_WIDGET (hc->dayslabel[i]), 50, 20);    

    hc->daysevent[i]=gtk_event_box_new ();
    gtk_widget_set_events (hc->daysevent[i], GDK_BUTTON_PRESS_MASK);
    gtk_widget_modify_bg (hc->daysevent[i], GTK_STATE_NORMAL, &color);

    hc->daystooltip[i] = gtk_tooltips_new  ();
    gtk_tooltips_set_tip (GTK_TOOLTIPS (hc->daystooltip[i]), hc->daysevent[i], "", "");
    gtk_tooltips_disable (GTK_TOOLTIPS (hc->daystooltip[i]));

    gtk_container_add (GTK_CONTAINER (hc->daysevent[i]), hc->dayslabel[i]); 
    g_signal_connect (G_OBJECT (hc->daysevent[i]), "button_press_event", 
		      G_CALLBACK (day_button_press_callback), hc);

    gtk_table_attach_defaults (GTK_TABLE (hc->table), hc->daysevent[i] , 
			       i%7, i%7+1, i/7, i/7+1 );    

    gtk_widget_show ( hc->dayslabel[i] );
    gtk_widget_show ( hc->daysevent[i] );

  }

}


void
create_heading ( Hebcalendar *hc )
{

  int i;
  int signal;

  /* creating labels */
  hc->headmonthlabel=gtk_label_new (""); 
  gtk_widget_show (hc->headmonthlabel);
  gtk_widget_set_size_request     (GTK_WIDGET (hc->headmonthlabel), 150, 20);

  hc->headyearlabel=gtk_label_new (""); 
  gtk_widget_show (hc->headyearlabel);
  gtk_widget_set_size_request     (GTK_WIDGET (hc->headyearlabel), 150, 20);

  /* creating arrows */
  hc->monthchange = gtk_hbox_new  (FALSE, 5);  
  for (i=0; i<4; i++)
  {
    switch (i) {
      case 0: 
	hc->montharrow[i]= gtk_image_new_from_stock ( "gtk-goto-first", 
				     GTK_ICON_SIZE_SMALL_TOOLBAR);
                                            
	signal=YEAR_PREV_SIGNAL;
	break;
      case 1:
	hc->montharrow[i]= gtk_image_new_from_stock ( "gtk-go-back", 
				     GTK_ICON_SIZE_SMALL_TOOLBAR);

	signal=MONTH_PREV_SIGNAL;
	break;
      case 2:
	hc->montharrow[i]= gtk_image_new_from_stock ( "gtk-go-forward", 
				     GTK_ICON_SIZE_SMALL_TOOLBAR);

	signal=MONTH_NEXT_SIGNAL;
	break;
      case 3:
	hc->montharrow[i]= gtk_image_new_from_stock ( "gtk-goto-last", 
				     GTK_ICON_SIZE_SMALL_TOOLBAR);

	signal=YEAR_NEXT_SIGNAL;
	break;
    } 

    gtk_widget_show ( hc->montharrow[i] );

    hc->montharrowbox[i] =  gtk_event_box_new ( );
    gtk_container_add (GTK_CONTAINER (hc->montharrowbox[i]), hc->montharrow[i]);

    g_signal_connect (G_OBJECT (hc->montharrowbox[i]),"button_press_event" , 
 		    G_CALLBACK (month_button_press_callback), hc);
    g_object_set_data (G_OBJECT (hc->montharrowbox[i]), "signal_id", 
		    GINT_TO_POINTER (signal));

    gtk_widget_show ( hc->montharrowbox[i] );
    gtk_box_pack_start (GTK_BOX (hc->monthchange),
			hc->montharrowbox[i], FALSE, FALSE,0);

  }

  gtk_widget_show (hc->monthchange);

  /* create the headings */
  hc->heading = gtk_hbox_new  (FALSE, 0);  
  gtk_box_pack_start (GTK_BOX (hc->heading),hc->headmonthlabel,TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hc->heading),hc->monthchange,TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hc->heading),hc->headyearlabel,TRUE, FALSE, 0);

}


void
create_hebdate (  Hebcalendar *hc )
{

  // create the current date button
  hc->tocurrentdate = gtk_image_new_from_stock ( "gtk-jump-to", 
						 GTK_ICON_SIZE_SMALL_TOOLBAR);
  hc->tocurrentdatebox = gtk_event_box_new ( );
  gtk_container_add (GTK_CONTAINER (hc->tocurrentdatebox), hc->tocurrentdate);
  g_signal_connect (G_OBJECT (hc->tocurrentdatebox),"button_press_event" , 
 		    G_CALLBACK (to_current_date_callback), hc);

  gtk_widget_show (hc->tocurrentdate);
  gtk_widget_show (hc->tocurrentdatebox);

  // create the hebrew month year, full date
  hc->texthebdate=gtk_entry_new ();
  gtk_entry_set_editable (GTK_ENTRY (hc->texthebdate), FALSE);
  gtk_widget_show (hc->texthebdate);

  hc->hebmonthlabel=gtk_label_new (""); 
  gtk_widget_show (hc->hebmonthlabel);

  hc->hebyearlabel=gtk_label_new (""); 
  gtk_widget_show (hc->hebyearlabel);

  // pack it
  hc->hebdate = gtk_hbox_new  (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hc->hebdate),hc->tocurrentdatebox,FALSE,FALSE, 5);
  gtk_box_pack_start (GTK_BOX (hc->hebdate),hc->texthebdate, FALSE, FALSE, 5);
  gtk_box_pack_end (GTK_BOX (hc->hebdate),hc->hebyearlabel, FALSE, FALSE, 5);
  gtk_box_pack_end (GTK_BOX (hc->hebdate),hc->hebmonthlabel, FALSE, FALSE, 5);
 
}



/*-----------------------------------------------------------------------------

hebrew calendar widget class function and definition

-----------------------------------------------------------------------------*/

static void hebcalendar_class_init          (HebcalendarClass *klass);
static void hebcalendar_init                (Hebcalendar      *hc);
static void hebcalendar_get_arg  (GtkObject *object, GtkArg *arg, guint arg_id);
static void hebcalendar_set_arg  (GtkObject *object, GtkArg *arg, guint arg_id);

static guint hebcalendar_signals[LAST_SIGNAL] = { 0 };

GType
hebcalendar_get_type (void)
{
  static GType hcal_type = 0;

  
  if (!hcal_type)
    {
      static const GTypeInfo heb_info =
      {
	sizeof (HebcalendarClass),
	NULL, /* base_init */
        NULL, /* base_finalize */
	(GClassInitFunc) hebcalendar_class_init,
        NULL, /* class_finalize */
	NULL, /* class_data */
        sizeof (Hebcalendar),
	0,
	(GInstanceInitFunc) hebcalendar_init,
      };

      hcal_type = g_type_register_static (GTK_TYPE_VBOX, 
		     "Hebcalendar", 
		     &heb_info, 
#ifdef __cplusplus
		     GTypeFlags (0)
#else
		     0
#endif
		     );

    }

  return hcal_type;
}

static void
hebcalendar_class_init (HebcalendarClass *klass)
{
  
  GtkObjectClass *object_class;

  object_class=(GtkObjectClass*) klass;

  hebcalendar_signals[DAY_SELECTED_SIGNAL] = g_signal_new ("day-selected",
          G_TYPE_FROM_CLASS (klass),
#ifdef __cplusplus  
	  GSignalFlags (G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION),
#else
	  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
#endif
	  G_STRUCT_OFFSET (HebcalendarClass, hebcalendar),
          NULL, 
          NULL,                
	  g_cclosure_marshal_VOID__VOID,
          G_TYPE_NONE, 0);

  
  hebcalendar_signals[MONTH_CHANGED_SIGNAL] = g_signal_new ("month-changed",
          G_TYPE_FROM_CLASS (klass),
#ifdef __cplusplus  
	  GSignalFlags (G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION),
#else
	  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
#endif
	  G_STRUCT_OFFSET (HebcalendarClass, hebcalendar),
          NULL, 
          NULL,                
	  g_cclosure_marshal_VOID__VOID,
          G_TYPE_NONE, 0);


  
  hebcalendar_signals[MONTH_PREV_SIGNAL] = g_signal_new ("prev-month",
          G_TYPE_FROM_CLASS (klass),
#ifdef __cplusplus  
	  GSignalFlags (G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION),
#else
	  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
#endif
	  G_STRUCT_OFFSET (HebcalendarClass, hebcalendar),
          NULL, 
          NULL,                
	  g_cclosure_marshal_VOID__VOID,
          G_TYPE_NONE, 0);

  
  hebcalendar_signals[MONTH_NEXT_SIGNAL] = g_signal_new ("next-month",
          G_TYPE_FROM_CLASS (klass),
#ifdef __cplusplus  
	  GSignalFlags (G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION),
#else
	  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
#endif
	  G_STRUCT_OFFSET (HebcalendarClass, hebcalendar),
          NULL, 
          NULL,                
	  g_cclosure_marshal_VOID__VOID,
          G_TYPE_NONE, 0);

  
  hebcalendar_signals[YEAR_PREV_SIGNAL] = g_signal_new ("prev-year",
          G_TYPE_FROM_CLASS (klass),
#ifdef __cplusplus  
	  GSignalFlags (G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION),
#else
	  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
#endif
	  G_STRUCT_OFFSET (HebcalendarClass, hebcalendar),
          NULL, 
          NULL,                
	  g_cclosure_marshal_VOID__VOID,
          G_TYPE_NONE, 0);

  
  hebcalendar_signals[YEAR_NEXT_SIGNAL] = g_signal_new ("next-year",
          G_TYPE_FROM_CLASS (klass),
#ifdef __cplusplus  
	  GSignalFlags (G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION),
#else
	  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
#endif
	  G_STRUCT_OFFSET (HebcalendarClass, hebcalendar),
          NULL, 
          NULL,                
	  g_cclosure_marshal_VOID__VOID,
          G_TYPE_NONE, 0);

  

  gtk_object_add_arg_type("Hebcalendar::day",
                          GTK_TYPE_UINT, 
                          GTK_ARG_READWRITE, 
                          ARG_DAY);

  gtk_object_add_arg_type("Hebcalendar::month",
                          GTK_TYPE_UINT, 
                          GTK_ARG_READWRITE, 
                          ARG_MONTH);

  gtk_object_add_arg_type("Hebcalendar::year",
                          GTK_TYPE_UINT, 
                          GTK_ARG_READWRITE, 
                          ARG_YEAR);

  gtk_object_add_arg_type("Hebcalendar::currentdaycolor",
                          GTK_TYPE_STRING, 
                          GTK_ARG_READWRITE, 
                          ARG_CURRENT_DAY_COLOR);

  gtk_object_add_arg_type("Hebcalendar::eventhagcolor",
                          GTK_TYPE_STRING, 
                          GTK_ARG_READWRITE, 
                          ARG_EVENT_COLOR_HAG);

  gtk_object_add_arg_type("Hebcalendar::eventhagdaycolor",
                          GTK_TYPE_STRING, 
                          GTK_ARG_READWRITE, 
                          ARG_EVENT_COLOR_HAGDAY);

  object_class->get_arg = hebcalendar_get_arg;
  object_class->set_arg = hebcalendar_set_arg;

}


static void
hebcalendar_init (Hebcalendar *hc)
{
  GtkWidget *sep1, *sep2;
  int i;

  sep1= gtk_hseparator_new ();
  sep2= gtk_hseparator_new ();
  

  create_heading  (hc);
  create_daynames (hc);
  create_table    (hc);
  create_hebdate  (hc);

  gtk_box_pack_start (GTK_BOX (hc),hc->heading , FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hc),hc->daynames, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hc), sep1, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hc),hc->table, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hc), sep2, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hc),hc->hebdate, FALSE, FALSE, 0);

  gtk_widget_show ( hc->heading );
  gtk_widget_show ( hc->daynames );
  gtk_widget_show ( hc->table );
  gtk_widget_show ( hc->hebdate );

  gtk_widget_show ( sep1 );
  gtk_widget_show ( sep2 );

#ifdef __cplusplus
  hc->flags =  HebCalendarDisplayOptions (
          HEB_CALENDAR_SHOW_HEADING |
          HEB_CALENDAR_SHOW_DAY_NAMES |
          HEB_CALENDAR_SHOW_HEB_DATE ) ;
#else
  hc->flags = 
          HEB_CALENDAR_SHOW_HEADING |
          HEB_CALENDAR_SHOW_DAY_NAMES |
    HEB_CALENDAR_SHOW_HEB_DATE ;
#endif


  /* fill colors table from constants */
  g_snprintf (hc->currentdaycolor, 8, "%s", currentcolor);
  g_snprintf (hc->eventcolors[0], 8, "%s", "white");
  g_snprintf (hc->eventcolors[1], 8, "%s", hagcolor);
  g_snprintf (hc->eventcolors[2], 8, "%s", hagdaycolor);
  //  g_snprintf (hc->eventcolors[3], 8, "%s", userevent);

  for (i=0;i<32;i++)
    hc->daysmarkday[i]=0;

  setcurrentdate (hc);

  hebsetmonth (hc);

}


GtkWidget*
hebcalendar_new ()
{
  return GTK_WIDGET (g_object_new (hebcalendar_get_type (), NULL));
}



static void
hebcalendar_set_arg (GtkObject    *object,
                       GtkArg       *arg,
                       guint         arg_id)
{
  Hebcalendar *hc;

  g_return_if_fail (HEBCALENDAR (object));

  hc=HEBCALENDAR (object);

  switch (arg_id)
    {
    case ARG_DAY:
      hc->hm.gmday=GTK_VALUE_UINT (*arg);
      break;
    case ARG_MONTH:
      hc->hm.gmonth=GTK_VALUE_UINT (*arg);
      break;
    case ARG_YEAR:
      hc->hm.gyear=GTK_VALUE_UINT (*arg);     
      break;
    case ARG_CURRENT_DAY_COLOR:
      g_snprintf (hc->currentdaycolor, 8, "%s", GTK_VALUE_STRING (*arg));
      break;
    case ARG_EVENT_COLOR_HAG:
      g_snprintf (hc->eventcolors[1], 8, "%s", GTK_VALUE_STRING (*arg));
      break;
    case ARG_EVENT_COLOR_HAGDAY:
      g_snprintf (hc->eventcolors[2], 8, "%s", GTK_VALUE_STRING (*arg));
      break;
    default:
      break;
    }
}

static void
hebcalendar_get_arg (GtkObject    *object,
                       GtkArg       *arg,
                       guint         arg_id)
{
  Hebcalendar *hc;

  g_return_if_fail (HEBCALENDAR (object));

  hc=HEBCALENDAR (object);
  
  switch (arg_id)
    {
    case ARG_DAY:
      GTK_VALUE_UINT (*arg) = hc->hm.gmday;
      break;
    case ARG_MONTH:
      GTK_VALUE_UINT (*arg) = hc->hm.gmonth;
      break;
    case ARG_YEAR:
      GTK_VALUE_UINT (*arg) = hc->hm.gyear;
      break;
    case ARG_CURRENT_DAY_COLOR:
      GTK_VALUE_STRING (*arg) = g_strdup_printf ("%s", hc->currentdaycolor);
      break;
    case ARG_EVENT_COLOR_HAG:
      GTK_VALUE_STRING (*arg) = g_strdup_printf ("%s", hc->eventcolors[1]);
      break;
    case ARG_EVENT_COLOR_HAGDAY:
      GTK_VALUE_STRING (*arg) = g_strdup_printf ("%s", hc->eventcolors[2]);
      break;
    default:
      arg->type = GTK_TYPE_INVALID;
      break;
    }
}    


/*-----------------------------------------------------------------------------

callbacks function

-----------------------------------------------------------------------------*/


static gboolean
month_button_press_callback (GtkObject *eventbox, 
                        GdkEventButton *event,
                         Hebcalendar *hc)
{
  int sig;
  int month, year;

  sig=GPOINTER_TO_INT (g_object_get_data  (G_OBJECT (eventbox), "signal_id"));

  month = hc->hm.gmonth;
  year = hc->hm.gyear;

  switch (sig) {
    case MONTH_NEXT_SIGNAL:
      month++; 
      if ( month > 12 ) {
	month = 1;	
	if ( year < 5000 )
	  year++;
      }

    break;

    case MONTH_PREV_SIGNAL:
      month--;
      if ( month < 1 ) {
	month = 12;	
	if ( year > 1000 )
	  year--;
      }  

    break;

    case YEAR_NEXT_SIGNAL:
      if ( year < 5000 )
	  year++;   

    break;
    case YEAR_PREV_SIGNAL:
      if (year > 1000 )
	  year--;

    break;


  } ;

  hebmonth_init (&hc->hm, hc->hm.gmday, month, year);
      
  g_signal_emit (G_OBJECT (hc), 
         hebcalendar_signals[sig], 0);

  g_signal_emit (G_OBJECT (hc), 
         hebcalendar_signals[MONTH_CHANGED_SIGNAL], 0);

  g_signal_emit (G_OBJECT (hc), 
         hebcalendar_signals[DAY_SELECTED_SIGNAL], 0);

  hebsetmonth (hc); 

  return TRUE;
}


static gboolean
day_button_press_callback (GtkObject *eventbox, 
                        GdkEventButton *event,
                         Hebcalendar *hc)
{
  int dayint;

  dayint=GPOINTER_TO_INT (g_object_get_data  (G_OBJECT (eventbox), "dayint"));
  if (!dayint)
    return TRUE;
 
  if (hc->hm.gmday!=dayint) {
    hebmonth_init (&hc->hm, dayint, hc->hm.gmonth, hc->hm.gyear);
  
    g_signal_emit (G_OBJECT (hc), 
         hebcalendar_signals[DAY_SELECTED_SIGNAL], 0);

    hebsetmonth (hc);

  }

  return TRUE;
}

static gboolean
to_current_date_callback (GtkObject *eventbox, 
                        GdkEventButton *event,
                         Hebcalendar *hc)
{

  setcurrentdate (hc);
  g_signal_emit (G_OBJECT (hc), 
         hebcalendar_signals[DAY_SELECTED_SIGNAL], 0);

  hebsetmonth (hc);

  return TRUE;

}


/*-----------------------------------------------------------------------------

external functions

-----------------------------------------------------------------------------*/


void
heb_calendar_select_month       (Hebcalendar *hc,
                                             guint month,
                                             guint year)
{

  g_return_if_fail (HEBCALENDAR (hc));
  hebmonth_init (&hc->hm, hc->hm.gmday, month, year);

  g_signal_emit (G_OBJECT (hc), 
         hebcalendar_signals[MONTH_CHANGED_SIGNAL], 0);

  hebsetmonth (hc);

}


void
heb_calendar_select_day         (Hebcalendar *hc,
                                             guint day)
{

  g_return_if_fail (HEBCALENDAR (hc));
  hebmonth_init (&hc->hm, day, hc->hm.gmonth, hc->hm.gyear);
  
  g_signal_emit (G_OBJECT (hc), 
         hebcalendar_signals[DAY_SELECTED_SIGNAL], 0);

  hebsetmonth (hc);

}


void        
heb_calendar_get_date           (Hebcalendar *hc,
                                             guint *year,
                                             guint *month,
                                             guint *day)
{
  g_return_if_fail (HEBCALENDAR (hc));

  *year=hc->hm.gyear;
  *month=hc->hm.gmonth;
  *day=hc->hm.gmday;

}


void        
heb_calendar_set_date           (Hebcalendar *hc)
{

  g_return_if_fail (HEBCALENDAR (hc));
  hebsetmonth (hc);

}


void        
heb_calendar_set_current_date           (Hebcalendar *hc)
{

  g_return_if_fail (HEBCALENDAR (hc));
  setcurrentdate (hc);
  hebsetmonth (hc);

}


void        
heb_calendar_set_display_options (Hebcalendar *hc,
				  HebCalendarDisplayOptions flags)
{

  g_return_if_fail (HEBCALENDAR (hc));

  hc->flags=flags;

  if (flags & HEB_CALENDAR_SHOW_HEADING)
    gtk_widget_show (hc->heading);
  else
    gtk_widget_hide (hc->heading);

  if (flags & HEB_CALENDAR_SHOW_DAY_NAMES)
    gtk_widget_show (hc->daynames);
  else
    gtk_widget_hide (hc->daynames);

  if (flags & HEB_CALENDAR_SHOW_HEB_DATE)
    gtk_widget_show (hc->hebdate);
  else
    gtk_widget_hide (hc->hebdate);


}


void  
heb_calendar_get_display_options (Hebcalendar *hc,
				  HebCalendarDisplayOptions *flags)
{

  g_return_if_fail (HEBCALENDAR (hc));
  *flags=hc->flags;

}

void  
heb_calendar_set_markup 	(Hebcalendar *hc,
				  int day)
{

  g_return_if_fail (HEBCALENDAR (hc));
  g_return_if_fail (day>0 && day<32);

  hc->daysmarkday[day - 1] = 1;

}

void  
heb_calendar_unset_markup 	(Hebcalendar *hc,
				  int day)
{

  g_return_if_fail (HEBCALENDAR (hc));
  g_return_if_fail (day>0 && day<32);

  hc->daysmarkday[day - 1]  = 0;

}


void  
heb_calendar_unset_all_markup 	(Hebcalendar *hc)
{
  int i;

  g_return_if_fail (HEBCALENDAR (hc));

  for (i=0;i<31;i++)
    hc->daysmarkday[i]=0;
  

}



void  
heb_calendar_set_all_markup 	(Hebcalendar *hc, 
				 int month[])
{
  int i;

  g_return_if_fail (HEBCALENDAR (hc));

  for (i=0;i<31;i++)
    hc->daysmarkday[i]=month[i+1]; 

}


void
heb_calnedar_get_absolute_date (Hebcalendar *hc, 
				int *ad)
{
  g_return_if_fail (HEBCALENDAR (hc));
  *ad=hc->hm.absolutedate ;
}

