/*------------------------------------------------------------------------------
Hebrew Calendar is a simple program which show the hebrew dates and holydays
of the gregorian calendar

Copyright (C) 2005  Asaf Shachar <asafsh1@users.sourceforge.net> 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
------------------------------------------------------------------------------*/



#ifndef __HEBCALENDAR_H__
#define __HEBCALENDAR_H__


#include <glib.h>
#include <glib-object.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtktooltips.h>
#include "hebmonth.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define HEBCALENDAR_TYPE            (hebcalendar_get_type ())
#define HEBCALENDAR(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), HEBCALENDAR_TYPE, Hebcalendar))
#define HEBCALENDAR_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), HEBCALENDAR_TYPE, HebcalendarClass))
#define IS_HEBCALENDAR(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), HEBCALENDAR_TYPE))
#define IS_HEBCALENDAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), HEBCALENDAR_TYPE))

typedef enum
{
  HEB_CALENDAR_SHOW_HEADING		= 1 << 0,
  HEB_CALENDAR_SHOW_DAY_NAMES		= 1 << 1,
  HEB_CALENDAR_SHOW_HEB_DATE		= 1 << 2
} HebCalendarDisplayOptions;

typedef struct _Hebcalendar       Hebcalendar;
typedef struct _HebcalendarClass  HebcalendarClass;
typedef struct Hebmonth           hebmonth;


struct _Hebcalendar
{
  GtkVBox vbox;

  hebmonth hm;


  GtkWidget   *heading;
  GtkWidget   *daynames;
  GtkWidget   *table;
  GtkWidget   *hebdate;
  
  GtkWidget   *montharrow[4];
  GtkWidget   *montharrowbox[4];
  GtkWidget   *monthchange;
  GtkWidget   *weekdays[7];

  GtkWidget   *hebmonthlabel;
  GtkWidget   *hebyearlabel;
  GtkWidget   *texthebdate;
  GtkWidget   *tocurrentdate;
  GtkWidget   *tocurrentdatebox;

  GtkWidget   *headmonthlabel;
  GtkWidget   *headyearlabel;

  GtkWidget   *dayslabel[42];
  GtkWidget   *daysevent[42];
  GtkTooltips *daystooltip[42];
  guint       daysmarkday[32];

  HebCalendarDisplayOptions flags;

  gchar       currentdaycolor[8];
  gchar       eventcolors[4][8];

};

struct _HebcalendarClass
{
  GtkVBoxClass parent_class;

  void (* hebcalendar) (Hebcalendar *hc);
  void (* get_arg) (GtkObject *object, GtkArg *arg, guint arg_id);
  void (* set_arg) (GtkObject *object, GtkArg *arg, guint arg_id);

};

GType          hebcalendar_get_type        (void);
GtkWidget*     hebcalendar_new             (void);

void
heb_calendar_select_month       (Hebcalendar *hc,
                                 guint month,
				 guint year);
void
heb_calendar_select_day         (Hebcalendar *hc,
				 guint day);

void        
heb_calendar_get_date           (Hebcalendar *hc,
                                 guint *year,
                                 guint *month,
				 guint *day);

void        
heb_calendar_set_date           (Hebcalendar *hc);


void        
heb_calendar_set_current_date   (Hebcalendar *hc);


void        
heb_calendar_set_display_options (Hebcalendar *hc,
				  HebCalendarDisplayOptions flags);

void  
heb_calendar_get_display_options (Hebcalendar *hc,
				  HebCalendarDisplayOptions *flags);

void  
heb_calendar_set_markup 	(Hebcalendar *hc,
				  int day);

void  
heb_calendar_unset_markup 	(Hebcalendar *hc,
				  int day);

void  
heb_calendar_unset_all_markup 	(Hebcalendar *hc);


void  
heb_calendar_set_all_markup 	(Hebcalendar *hc, 
				 int month[]);


void
heb_calnedar_get_absolute_date (Hebcalendar *hc, 
				int *ad);


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __HEBCALENDAR_H__ */
