/*------------------------------------------------------------------------------
Hebrew Calendar is a simple program which show the hebrew dates and holydays
of the gregorian calendar

Copyright (C) 2005  Asaf Shachar <asafsh1@users.sourceforge.net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
------------------------------------------------------------------------------*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef WITH_LIBHDATE
#include <hdatepp.h>
using namespace hdate;
#endif

#include "hebmonth.h"
#include "calendar.h"
#include "hagim.h"
#include "tohebalef.h"
#include "tohebmonth.h"

static Hagim hag_year;


void hebmonth_init (Hebmonth *hm, int d, int m, int y) { 

    hm->gmday=d;
    hm->gmonth=m;

    //cache the year holidays if needed
    if (hm->gyear!=y) {
    	if (hag_year.iscached())
    		hag_year.empty_cache ();
    	hag_year.cache_year (y);
    }
	
    hm->gyear=y;

    // if current day exceeds last day ajust to last day of month
    hm->numofdays=LastDayOfGregorianMonth(m, y);
    if (d > hm->numofdays)
	hm->gmday=hm->numofdays;

    // get the absolute date of first day of gregorian month
    GregorianDate g(m,1,y);
    int a = g;
    hm->absolutedate=a;
    hm->firstdow=a%7;

#ifdef WITH_LIBHDATE

    Hdate h;

    h.set_gdate (1, m, y);   	
    hm->julianday=h.get_julian ();

#else

    // get the hebrew date of the gergorian first day of month
    HebrewDate h(a);
    hm->month=h.GetMonth ();
    hm->year=h.GetYear ();
    hm->leapyear=HebrewLeapYear (h.GetYear());

#endif
    
}

// fill buf with current hebrew date
void heb_date (Hebmonth *hm, char *buf)
{

#ifdef WITH_LIBHDATE
  Hdate h;

	// set the date
  h.set_jd (hm->julianday+hm->gmday - 1);

	// copy the date FALSE - in israel TRUE - use short notation
  strcpy (buf, h.get_format_date (0));

#else

  HebrewDate h(hm->absolutedate + hm->gmday - 1);

  string d=tohebalefgeresh (h.GetDay ()) ;
  d+= " ";

  d+=tohebalef (2); // bet
  d+=tohebmonth (h.GetMonth (), hm->leapyear) ;
  d+= " ";

  d+=tohebalef (h.GetYear()) ;	

  d.copy (buf, string::npos);
  buf[d.length()]=0;

#endif

}

// fill buf with the day of month
void heb_day (Hebmonth *hm, int n, char *buf)
{

#ifdef WITH_LIBHDATE

  Hdate h;
  // set the date
  h.set_jd (hm->julianday+n);

  //string d=tohebalef (h.get_hday ());
  strcpy (buf, h.get_hebrew_day_string ());
#else

  HebrewDate h(hm->absolutedate + n);
  string d=tohebalef (h.GetDay());
  d.copy (buf, string::npos);
  buf[d.length()]=0;

#endif


}

// fill buf with  the heb month start - heb month end 
void heb_monthyear (Hebmonth *hm, char *buf)
{

#ifdef WITH_LIBHDATE

  Hdate h;

	// set the date
  h.set_jd (hm->julianday);
	// copy the this month to buf, FALSE - dont use abv'
  strcpy (buf, h.get_hebrew_month_string (false));

  h.set_jd (hm->julianday + hm->numofdays-1);
  strcat (buf, " - ");
  strcat (buf, h.get_hebrew_month_string (false));

#else

  string d=tohebmonth (hm->month, hm->leapyear) + " - ";

  HebrewDate h1(hm->absolutedate + hm->numofdays - 1);
  d+=tohebmonth (h1.GetMonth (), hm->leapyear);

  d.copy (buf, string::npos);
  buf[d.length()]=0;

#endif

}

// fill buf with hebrew year
void heb_year (Hebmonth *hm, char *buf)
{

#ifdef WITH_LIBHDATE
  
// for date conversions
  Hdate h;

	// set the date
  h.set_jd (hm->julianday);

	// copy the year to buf
  strcpy (buf, h.get_hebrew_year_string ());

#else

  string d=tohebalef (hm->year);

  d.copy (buf, string::npos);
  buf[d.length()]=0;

#endif

}


// fill buf with hag short descirption (only name not in use)
int heb_hag_short (Hebmonth *hm, int n, char *buf)
{

  int rc=0;
  char hagdesc[100];
  char haghdesc[100];
  int daynum;
  int code;	

  int ad=hm->absolutedate + n;

  if (hag_year.get_hag (ad, hagdesc, haghdesc, &code, &daynum)) {

#ifdef WITH_LIBHDATE
	string d=hagdesc;
#else
	string d=haghdesc;
#endif

	d.copy (buf, string::npos);
  	buf[d.length()]=0;
	rc=daynum+1;
  }
  return rc;
}

// fill buf with hag full description
// return : 0: no hag, 1: erev hag >1: hag for color
int heb_hag (Hebmonth *hm, int n, char *buf)
{
  int rc=0;
  char hagdesc[100];
  char haghdesc[100];
  int daynum;
  int code;	

  int ad=hm->absolutedate + n;

  if (hag_year.get_hag (ad, hagdesc, haghdesc, &code, &daynum)) {

#ifdef WITH_LIBHDATE
	string d=hagdesc;
#else
	string d=haghdesc;
#endif

	tohebhag (d, code, daynum);
	d.copy (buf, string::npos);
  	buf[d.length()]=0;

	rc=daynum+1;
  }

  return rc;
}

// fill buf with the day alef bet (in gimetria)
void heb_alef (int n, char *buf)
{
  string d=tohebalef (n);
  d.copy (buf, string::npos);
  buf[d.length()]=0;
}

