 /*------------------------------------------------------------------------------
Hebrew Calendar is a simple program which show the hebrew dates and holydays
of the gregorian calendar

Copyright (C) 2005  Asaf Shachar <asafsh1@users.sourceforge.net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
------------------------------------------------------------------------------*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>
#include <libintl.h>
#include "callbacks.h"
#include "hebcalendar.h"
#include "eventsgui.h"
#include "dlgmonthyear.h"
#include "dialogs.h"
#include "utils.h"
#include "gtklocale.h"

static GtkWidget *window=NULL;


static gchar *calendarcolorname[3]={"currentdaycolor",
			 "eventhagcolor",
			 "eventhagdaycolor"}; 


static void
get_colors (GtkWidget *hcal)
{

  char *bgcolors[3];

  g_object_get (HEBCALENDAR (hcal), 
		calendarcolorname[0],  &bgcolors[0], 
		calendarcolorname[1],  &bgcolors[1], 
		calendarcolorname[2],  &bgcolors[2], 
		NULL);
  

  utils_get_colors (bgcolors);

  for (int i=0;i<3;i++) {
    g_object_set (HEBCALENDAR (hcal), calendarcolorname[i], bgcolors[i], NULL);
    g_free (bgcolors[i]);	
  }

  //apply changes 
  heb_calendar_set_date (HEBCALENDAR (hcal));

}



gboolean 
delete_event( GtkWidget *widget,
              GdkEvent  *event,
              gpointer   data )
{

  utils_close_db ();

  gtk_main_quit ();
  
  return FALSE;

}


static void 
menuitem_about (GtkWidget *widget, gpointer data)
{

  dialog_about ();

}

static void 
menuitem_show (GtkWidget *widget, gpointer data)
{
 

  if (data==NULL)
    return;

  if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))	
    gtk_widget_show (GTK_WIDGET (data)); 
  else
    gtk_widget_hide (GTK_WIDGET (data)); 

}



static void 
menuitem_pick_colors (gpointer data)
{
  
  char *bgcolors[3];

  g_object_get (HEBCALENDAR (data), 
		calendarcolorname[0],  &bgcolors[0], 
		calendarcolorname[1],  &bgcolors[1], 
		calendarcolorname[2],  &bgcolors[2], 
		NULL);
  
  if (dialog_pick_colors (window, bgcolors)>0)
  {

  	g_object_set (HEBCALENDAR (data), 
		calendarcolorname[0],  bgcolors[0], 
		calendarcolorname[1],  bgcolors[1], 
		calendarcolorname[2],  bgcolors[2], 
		NULL);

  	//apply changes 
	heb_calendar_set_date (HEBCALENDAR (data));

	utils_set_colors (bgcolors);
  } 

  for (int i=0;i<3;i++) 
      g_free (bgcolors[i]);	

}


static void 
menuitem_export (gpointer data)
{
  
  int year,month,day;

  g_object_get (HEBCALENDAR (data),
		"year", &year,
		"month", &month,
		"day", &day,
		NULL); 

  dialog_export (window, year, month, day);

}


static GtkWidget *
create_menu_file (GtkWidget *hcal)
{
  GtkWidget *menu;
  GtkWidget *menu_items;
  GtkWidget *menu_quit;
  GtkWidget *root_menu;

  menu = gtk_menu_new ();		
  menu_items = gtk_menu_item_new_with_label (_("Export")); 
  g_signal_connect_swapped (G_OBJECT (menu_items), "activate",
                      G_CALLBACK (menuitem_export), 
                      gpointer (hcal));
 
  gtk_widget_show (menu_items);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_items);

  menu_items = gtk_separator_menu_item_new ();
  gtk_widget_show (menu_items);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_items);

  menu_quit = gtk_menu_item_new_with_label (_("Quit")); 
  g_signal_connect_swapped (G_OBJECT (menu_quit), "activate",
			      G_CALLBACK (gtk_widget_destroy),
                              G_OBJECT (window));

  gtk_widget_show (menu_quit);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_quit);

  root_menu = gtk_menu_item_new_with_label (_("File"));
  gtk_widget_show (root_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (root_menu), menu);

  return root_menu;

}


static GtkWidget *
create_menu_item_view (GtkWidget *menu, gchar *label, GtkWidget *widget, int checked)
{

  GtkWidget *menuitem;

  menuitem = gtk_check_menu_item_new_with_label (label); 

  if (checked)
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem), TRUE );

  g_signal_connect (G_OBJECT (menuitem), "activate",
                      G_CALLBACK (menuitem_show), 
                      gpointer (widget));
 
  gtk_widget_show (menuitem);

  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);

  return menuitem;
}





void
create_main_window ( )
{

  GtkWidget *hcalframe;
  GtkWidget *vbox;  

  GtkWidget *hcal;
  GtkWidget *data;
  GtkWidget *jumpto;	

  GtkWidget *menu_bar;
  GtkWidget *menu;

  GtkWidget *viewitem;
  GtkWidget *viewmenu;
  GtkWidget *viewrootmenu;
  GtkWidget *coloritem;
  
  GtkWidget *aboutitem;
  GtkWidget *helpmenu;
  GtkWidget *helprootmenu;

  hcal = hebcalendar_new ();

  g_signal_connect (G_OBJECT (hcal), "day-selected",
                    G_CALLBACK (hcal_day_selected), NULL);


  g_signal_connect (G_OBJECT (hcal), "month-changed",
                    G_CALLBACK (hcal_month_changed), NULL);


  gtk_widget_show (hcal);

  hcalframe= gtk_frame_new ("");
  gtk_container_add (GTK_CONTAINER (hcalframe), hcal);
  gtk_widget_show (hcalframe);

  data= create_data_widget ();
  jumpto=create_month_year (hcal);

  get_colors (hcal);

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  g_signal_connect (G_OBJECT (window), "destroy",
                    G_CALLBACK (delete_event), NULL);
  
  gtk_container_set_border_width (GTK_CONTAINER (window), 10);  
  gtk_window_set_title (GTK_WINDOW (window), _("Hebrew Calendar"));

  //create the menu
  menu_bar = gtk_menu_bar_new ();
  gtk_widget_show (menu_bar);
  menu = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (menu), menu_bar, FALSE, FALSE, 2);
  gtk_widget_show (menu);

  //create file menu
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), create_menu_file (hcal) );

  //create view menu
  viewmenu = gtk_menu_new ();
  create_menu_item_view (viewmenu, _("Show Events"), data, 1);
  create_menu_item_view (viewmenu, _("Show Jump To"), jumpto, 1);

  coloritem = gtk_menu_item_new_with_label (_("Pick Colors")); 
  gtk_widget_show (coloritem);
  gtk_menu_shell_append (GTK_MENU_SHELL (viewmenu), coloritem);
  g_signal_connect_swapped (G_OBJECT (coloritem), "activate",
                      G_CALLBACK (menuitem_pick_colors), 
                      gpointer (hcal));
 
  //attach view to menu bar
  viewrootmenu = gtk_menu_item_new_with_label (_("View"));
  gtk_widget_show (viewrootmenu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (viewrootmenu), viewmenu);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), viewrootmenu );


  //create about menu
  helpmenu = gtk_menu_new ();
  aboutitem = gtk_menu_item_new_with_label (_("About hcal-gtk...")); 
  gtk_widget_show (aboutitem);
  gtk_menu_shell_append (GTK_MENU_SHELL (helpmenu), aboutitem);
  g_signal_connect_swapped (G_OBJECT (aboutitem), "activate",
                      G_CALLBACK (menuitem_about), 
                      NULL);

  //attach help menu  
  helprootmenu = gtk_menu_item_new_with_label (_("Help"));
  gtk_widget_show (helprootmenu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (helprootmenu), helpmenu);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), helprootmenu );


  vbox=gtk_vbox_new (FALSE,0);
  gtk_box_pack_start (GTK_BOX (vbox), menu, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hcalframe, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), data, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), jumpto, FALSE, FALSE, 0);
  

  gtk_widget_show ( vbox );

  gtk_container_add (GTK_CONTAINER (window), vbox);


  utils_open_db ();

  //populate current day
  hcal_day_selected (hcal, NULL);
  hcal_month_changed (hcal, NULL);

  gtk_widget_show (window);
  
}

