/*------------------------------------------------------------------------------
Hebrew Calendar is a simple program which show the hebrew dates and holydays
of the gregorian calendar

Copyright (C) 2005  Asaf Shachar <asafsh1@users.sourceforge.net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
------------------------------------------------------------------------------*/

#include <glib.h>
#include <glib/gstdio.h>
#include <stdio.h>
#include <string.h>
#include "sqlitedb.h"


static const char *localdir=".hcal-gtk";
static const char *db="hcal.db";
static const char *conf="hcal.conf";


int 
utils_open_db ( )
{
  gchar *filename;
  int rc;

  filename=g_build_filename (g_get_home_dir (), localdir, db, NULL);
  
  rc=sqlite_open_db (filename);	  

  g_free (filename);
	
  return rc;

}


void 
utils_close_db ( )
{

  sqlite_close_db ( );

}



int 
create_install_dir (gchar *dirname)
{

    int rc;

    g_print ("creating %s\n", dirname);
    
    if (g_mkdir  (dirname,S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH  ) == 0) {
      g_print ("directory created OK\n");
      rc=1;
    } else {
      g_print ("cannot create %s\n", dirname);
      rc=0;
    } 

    return rc;

}


int 
copy_file (gchar *source, gchar *file)
{

    gchar *contents;
    gsize length;
    GError **error=NULL;
    FILE *out;
    int rc;

    g_print ("copying hcal data to %s\n", file);
    if (!g_file_get_contents (source,
                              &contents,
                              &length,
			      error)) {

      g_print ("cannot open source file%s\n", source);
      g_free (contents);
      return 0;
    }

    out=g_fopen (file, "w+");
    if (out!=NULL) {
      if (fwrite (contents, length, 1, out)) {
	printf ("copying %s OK\n", file);
	rc=1;
      } else {
	printf ("error writing file%s\n",file);
	rc=0;
      }
    } else {
	printf ("error creating file %s\n",file);
	rc=0;
    }

    g_free (contents);
    fclose (out);

    return rc;


}

void 
write_key_file (GKeyFile *kf, gchar *filename)
{
  gsize datasize;
  GError **error=NULL;
  gchar *data;
  FILE *out;

  data=g_key_file_to_data (kf,&datasize,error);
  if (error==NULL)
  {
    out=g_fopen (filename, "w+");
    if (out!=NULL) {
      if (!fwrite (data, datasize, 1, out)) 
	g_print ("Error writing configuration file\n");
    } else
	g_print ("Error oppening configuration file\n");		
  } else
   	g_print ("Error reading configuration file\n");			

  fclose (out);
  g_free (data);

}



int 
check_install_local ( )
{

  gchar *dirname, *confname, *sourceconfname, *dbname, *sorcedbname;
	
  dirname=g_build_filename (g_get_home_dir (),localdir,NULL);

  if (!g_file_test (dirname, G_FILE_TEST_EXISTS  ) ) {
    if (!create_install_dir (dirname)) {
      g_free (dirname);	
      return 0;
    }
  }
  g_free (dirname);	

  dbname=g_build_filename (g_get_home_dir (), localdir, db, NULL);
  if (!g_file_test (dbname, G_FILE_TEST_EXISTS  ) ) {
    sorcedbname=g_build_filename (PACKAGE_DATA_DIR, db, NULL);
    if (!copy_file (sorcedbname, dbname)) {
      g_free (sorcedbname);
      g_free (dbname);

      return 0;
    }
    g_free (sorcedbname);
  }
  g_free (dbname);

  confname=g_build_filename (g_get_home_dir (),localdir,conf,NULL);
  if (!g_file_test (confname, G_FILE_TEST_EXISTS  ) ) {
    sourceconfname=g_build_filename (PACKAGE_DATA_DIR,conf,NULL);	
    if (!copy_file (sourceconfname,confname)) {
      g_free (sourceconfname);
      g_free (confname);

      return 0;
    }
    g_free (sourceconfname);
  }
  g_free (confname);  
   


  return 1;

}



void 
utils_get_colors ( char *colorarr[3] )
{

  gchar *filename;
  GKeyFile *kf;
  GError **error=NULL;

  char *val;
  char *colorname[3]={"currentbgcolor", "hagbgcolor", "hagdaybgcolor"};
  int i;

  kf=g_key_file_new ();

  filename=g_build_filename (g_get_home_dir (),localdir, conf, NULL);

  if ( g_key_file_load_from_file    (kf,
				     filename,
                                     G_KEY_FILE_NONE,
 				    error) )
  {

    for (i=0;i<3;i++) {
      val=g_key_file_get_string    (kf,
                              "colors",
			      colorname[i],
                              error) ;

      if (val!=NULL && strlen (val)==7)
	snprintf (colorarr[i],8,"%s",val);
    }

  }


  g_key_file_free (kf);
  g_free (filename);

}

void 
utils_set_colors ( char *colorarr[3] )
{

  GKeyFile *kf;
  GError **error=NULL;
  gchar *filename;

  char *val;
  char *colorname[3]={"currentbgcolor", "hagbgcolor", "hagdaybgcolor"};
  int i;


  kf=g_key_file_new ();
  filename=g_build_filename (g_get_home_dir (), localdir, conf ,NULL);

  if ( g_key_file_load_from_file    (kf,
				     filename,
                                     G_KEY_FILE_KEEP_COMMENTS,
 				    error) )
  {

    for (i=0;i<3;i++) {
      g_key_file_set_string    (kf,
                              "colors",
			      colorname[i],
			      colorarr[i]
                              ) ;

    }

  }

  write_key_file (kf, filename);
  g_key_file_free (kf);
  g_free (filename);
}


